// Interactive Web-Based Pointillist Visualization of Hydrogenic Orbitals Using Jmol
//
// Shane P. Tully, Thomas M. Stitt, Robert D. Caldwell, Brian J. Hardock, 
// Robert M. Hanson, and Przemyslaw Maslak
//
// Department of Chemistry, the Pennsylvania State University, University
// Park, PA 16802, and Department of Chemistry, St. Olaf College, Northfield,
// MN 55057


// Shane Tully
// This file provides various convenience functions for jmol scripting.

// BH 11/27/2017 10:41:27 AM adapted for Jmol Unplugged. 

// Constants
var DEFAULT_PLANE_SIZE = 50;
var DEFAULT_PLANE = "XY";
var DEFAULT_CONTOUR_NUM_LINES = 20;

var DEFAULT_N_VALUE = 1;
var DEFAULT_L_VALUE = 0;
var DEFAULT_M_VALUE = 0;
var DEFAULT_Z_VALUE = 4;

var DEFAULT_CUTOFF = 0.015;

//var DEFAULT_ISOSURFACE_COLORPHASE = "darkgreen [x61AD3E]";
var DEFAULT_ISOSURFACE_COLORPHASE = "red blue";
var DEFAULT_WIREFRAME_COLORPHASE = "[255,100,0] [0,100,255]"

var DEFAULT_RESOLUTION = 7;
var DEFAULT_WIREFRAME_RESOLUTION = 1.5;
var DEFAULT_CROSSSECTION_CONTOUR_PLANE_RESOLUTION = 6; // BH 11/27/2017 10:41:15 AM was 5

var DEFAULT_NUM_POINTS = 5000;

var DEFAULT_PLANE_OPACITY = 0.5;
var DEFAULT_PLANE_COLOR = "gray";

var DEFAULT_AXES_DIAMETER = 2;

var DEFAULT_NODE_COLOR = "grey";
var DEFAULT_NODE_TRANSPARENCY = 25;

var TO_SCALE_ZOOM_LEVEL = 30;
var TO_SCALE_AXES_SCALE = 6;

var DEFAULT_SPINX = 10;
var DEFAULT_SPINY = 15;
var DEFAULT_SPINZ = 28;


// Globals
var curDotScale = 3;
var isToScale = true;
var curAxesDiameter = DEFAULT_AXES_DIAMETER;
var postCalcFunction = "";
var curOrbital = new Array(5);
var displayedAxesTicks = [false, false, false];

var jmol = "jmol"


function jmolScript(script) {
	console.log(script);
	Jmol.script(jmol, script);
}


function jmolScriptWait(script) {
	return Jmol.scriptWait(jmol, script);
}


function jmolGetPropertyAsArray(info) {
  return Jmol.getPropertyAsArray(jmol, info);
}


function jmolGetPropertyAsString(info) {
  return Jmol.getPropertyAsString(jmol, info);
}


function jmolSaveOrien(id) {
   Jmol.saveOrientation(jmol, id);
}


function jmolRestoreOrien(id) {
   Jmol.restoreOrientation(jmol, id);
}


function jmolInitEmptyApplet() {
   return "javascript jmolLoaded();"; 
}

function jmolLoaded() {
   // This function will be called when jmol reports that it is finished loading.
   // Do NOT call jmolScriptWait() in here! Only jmolScript(). jmolScriptWait() will cause the page to freeze
   jmolSetToScale(isToScale);
   jmolScript("frank off; set axesMolecular; set echo top right; font echo 12 sansSerif bold; " +
		          "axes on; axes 2; moveto 1 {1.5 0.2 1} 290 90");
}


function jmolGetZoomValue() {
   return jmolGetPropertyAsArray("orientationInfo").zoom;
}


function jmolGetIsosurfaceInfo() { // was "isosurfaceInfo" -- BH 8/2/2014 2:33:10 AM
	 return jmolGetPropertyAsArray("isosurfaceData");
}


function jmolGetN() {
   return ((curOrbital[0] == undefined) ? -1 : curOrbital[0]);
}


function jmolGetL() {
   return ((curOrbital[1] == undefined) ? -1 : curOrbital[1]);
}


function jmolGetM() {
   return ((curOrbital[2] == undefined) ? -1 : curOrbital[2]);
}


function jmolGetZ() {
   return ((curOrbital[3] == undefined) ? -1 : curOrbital[3]);
}


function jmolGetOrbitalName() {
   return ((curOrbital[4] == undefined) ? "N/A" : curOrbital[4]);
}


function jmolSetOrbitalValues(n, l, m, z) {
   if(!z)
      z = DEFAULT_Z_VALUE;

   // Store the parameters in the curOrbital array
   if(!isNaN(n) && !isNaN(l) && !isNaN(m)) {
      curOrbital[0] = n;
      curOrbital[1] = l;
      curOrbital[2] = m;
      curOrbital[3] = z;
      curOrbital[4] = getOrbitalName(n, l ,m);
   }
}


function jmolSetN(n) {
   if(!isNaN(n))
      curOrbital[0] = n;
   updateName();
}


function jmolSetL(l) {
   if(!isNaN(l))
      curOrbital[1] = l;
   updateName();
}


function jmolSetM(m) {
   if(!isNaN(m))
      curOrbital[2] = m;
   updateName();
}


function jmolSetZ(z) {
   curOrbital[3] = z;
}


function jmolSetToScale(isToScale) {
   this.isToScale = isToScale;

   // If to scale, the zoom should be 30. If not, the zoom 
   // will be adjusted when an isosurface is shown because 
   // it is based on n of the orbital displayed.
   if(isToScale) {
      jmolChangeZoom(TO_SCALE_ZOOM_LEVEL);
      jmolScaleAxes(TO_SCALE_AXES_SCALE);
      
      // The axis ticks scale needs changed as well
      if(displayedAxesTicks[0])
         jmolShowAxesTicks("x");
      if(displayedAxesTicks[1])
         jmolShowAxesTicks("y");
      if(displayedAxesTicks[2])
         jmolShowAxesTicks("z");
   } else
      setZoom();
}


function jmolChangeZoom(zoomValue) {
   if(!isNaN(zoomValue))
      jmolScript("zoom on; zoom " + zoomValue + ";");
}


function jmolDeleteAllIsosurfaces() {
  jmolScript("isosurface * delete; draw * delete;");
}


function jmolDeleteIsosurfaces(deletePlanes) {
   if(!deletePlanes)
      deletePlanes = true;

   jmolScript("isosurface curOrbital delete; isosurface crossSection* delete; isosurface contour* delete " + ((deletePlanes) ? "isosurface plane* delete" : "") + ";");
}


function jmolReset(reinit) {
   if(!reinit)
      reinit = false;

   jmolScript("zap; " + ((reinit) ? jmolInitEmptyApplet() : "") + ";");
}


function jmolRunScript(pathtoScript) {
   // TODO: check if script doesn't exist
   if(!pathtoScript)
      return -1;

   jmolScript("script " + pathtoScript + ";");
}


function jmolStopAllScripts() {
   jmolScript("exit");
}


function jmolSetPostCalcFunction(functionToCall) {
   postCalcFunction = functionToCall;
}

function toJmolTransparency(transparencyPercent) {
  return ((1 - (transparencyPercent/100))*8)|0;
}

function toJmolResolution(resolution) {
  return Math.max(resolution - curOrbital[0], 3);
}

 - curOrbital[0]
 
 
function jmolChangeTransparency(transparencyPercent) {
   // WARNING: This function is deprecated. Use jmolChangeTransparencyByID() instead.
   
   // Transparency values must be between 0 and 1 with 0 being opaque
   jmolScript("color isosurface translucent " + toJmolTransparency(transparencyPercent) + ";");
}


function jmolChangeTransparencyByID(transparencyPercent, isosurfaceID) {
  if(!isosurfaceID)
     isosurfaceID = "curOrbital";
    
  // Transparency values must be between 0 and 1 with 0 being opaque
  jmolScript("color $" + isosurfaceID + " translucent " + toJmolTransparency(transparencyPercent) + ";");
}


function jmolChangeBgColor(color) {
   jmolScript("background " + color + ";");
}


function jmolSetDotScale(dotScale) {
  if(!isNaN(dotScale))
    curDotScale = dotScale;

  jmolScript("set dotScale " + dotScale + ";");
}


function jmolMoveTo(direction, time) {
   if(direction == "RIGHT")
      coors = "0 1 0 -90";
   else if(direction == "LEFT")
      coors = "0 1 0 90";
   else if(direction == "TOP")
      coors = "1 0 0 90";
   else if(direction == "BOTTOM")
      coors = "1 0 0 -90";
   else if(direction == "FRONT")
      coors = "0 0 1 0";
   else if(direction == "BACK")
      coors = "0 1 0 180";

   jmolScript("moveto " + time + " " + coors + " " + ((isToScale) ? TO_SCALE_ZOOM_LEVEL : getZoomLevel()) + ";");
}


function jmolSpin(isSpinning, spinX, spinY, spinZ) {
   // Default spinX, spinY, and spinZ if not given
   if(!spinX)
      spinX = DEFAULT_SPINX;
   if(!spinY)
      spinY = DEFAULT_SPINY;
   if(!spinZ)
      spinZ = DEFAULT_SPINZ;

   jmolScript("set spinx " + spinX + "; set spiny " + spinY + "; set spinz " + spinZ + "; " + ((isSpinning) ? "spin on" : "spin off"));
}


function jmolSetAxesDiameter(diameter) {
  if(isNaN(diameter))
    curAxesDiameter = DEFAULT_AXES_DIAMETER;
  else
    curAxesDiameter = diameter;
    
  jmolScript("axes " + curAxesDiameter);
}


function jmolScaleAxes(scale) {
  jmolScript("axes scale " + scale);
}


function jmolToggleAxes(isOn) {
   jmolScript("axes " + ((isOn) ? "on" : "off") + ";");
}


function jmolShowAxesTicks(axis, major, minor, subminor) {
  if(!major) {
    if(!isToScale)
       major = getAxesTicksScale();
    else
       major = 5;
  }
  if(!minor)
    minor = 1;
  if(!subminor)
    subminor = 0;

  jmolScript("axes ticks " + axis + "{" + major + " " + minor + " " + subminor + "} format [\"%0.0f\"]");
  jmolSetAxesDiameter();

  // Update displayedAxesTicks
  if(axis.toLowerCase() == "x")
     displayedAxesTicks[0] = true;
  else if(axis.toLowerCase() == "y")
     displayedAxesTicks[1] = true;
  else if(axis.toLowerCase() == "z")
     displayedAxesTicks[2] = true;
}


function jmolHideAxesTicks(axis) {
  jmolScript("axes ticks " + axis + "{0 0 0}");
  jmolSetAxesDiameter();

  // Update displayedAxesTicks
  if(axis.toLowerCase() == "x")
     displayedAxesTicks[0] = false;
  else if(axis.toLowerCase() == "y")
     displayedAxesTicks[1] = false;
  else if(axis.toLowerCase() == "z")
     displayedAxesTicks[2] = false;
}


function jmolToggleBoundingBox(isOn) {
   jmolScript("set showBoundBox " ((isOn) ? "true" : "false") + ";");
}


function jmolSetBoundingBoxScale(scale) {
   if(isNaN(scale))
      scale == 1;

   jmolScript("boundbox scale " + scale + ";");
}


function jmolShowWireframe(cutoff, resolution, colorPhase) {
   if(isNaN(cutoff))
      cutoff = DEFAULT_CUTOFF;

   if(isNaN(resolution))
      resolution = DEFAULT_WIREFRAME_RESOLUTION;

   jmolDeleteIsosurfaces(false);
   // WARNING: isWireframeDisplayed() depends on "mesh nofill" being in the command below
   jmolScript(getDefaultIsosurfaceCommand(cutoff, resolution, true, colorPhase) + "; isosurface mesh nofill;" + " echo " + curOrbital[4] + "; javascript " + postCalcFunction + ";");
   setZoom();
}


function jmolShowCentileBoundary(cutoff, centileCutoff, resolution, colorPhase) {
     if(isNaN(cutoff))
      cutoff = DEFAULT_CUTOFF;

   if(isNaN(resolution))
      resolution = DEFAULT_WIREFRAME_RESOLUTION;

   if(!colorPhase)
      colorPhase = DEFAULT_WIREFRAME_COLORPHASE;

   jmolDeleteIsosurfaces(false);

   jmolShowIsosurfaceSkin(cutoff);
   jmolShowWireframe(centileCutoff, resolution, colorPhase);
}


function jmolShowSolidIsosurface(cutoff, resolution, colorPhase) {
   jmolDeleteIsosurfaces(false);
   jmolScript(getDefaultIsosurfaceCommand(cutoff, resolution, false, colorPhase) + "; echo " + curOrbital[4] + "; javascript " + postCalcFunction + ";");
   setZoom();
}


function jmolShowPointillist(dotScale, numPoints, cutoff, resolution) {
   // Update the curDotScale
   if(!isNaN(dotScale))
      curDotScale = dotScale;

   if(isNaN(numPoints))
      numPoints = DEFAULT_NUM_POINTS;

   jmolDeleteIsosurfaces(false);
   // WARNING: The isPointillistDisplayed() function depends on 'POINTS' being in this command
   jmolScript(getDefaultIsosurfaceCommand(cutoff, resolution) + " points " + numPoints + "; set dotscale " + curDotScale + "; echo " + curOrbital[4] + "; javascript " + postCalcFunction + ";");
   setZoom();
}


function jmolShowDensityPointillist(dotScale, numPoints, seed) {
     // Update the curDotScale
   if(!isNaN(dotScale))
      curDotScale = dotScale;

   if(isNaN(numPoints))
      numPoints = DEFAULT_NUM_POINTS;

   jmolScript("isosurface atomicOrbital " + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3] + " points " + numPoints + " " + seed + " map atomicOrbital "  + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3] + "; set dotscale " + curDotScale + "; echo " + curOrbital[4] + "; javascript " + postCalcFunction + ";");
   setZoom();
}


function jmolShowIsosurfaceSkin(cutoff) {
   // Old cutoff: 0.01
   if(isNaN(cutoff))
      cutoff = DEFAULT_CUTOFF;
   
   jmolScript("isosurface curOrbitalSkin cutoff " + cutoff + " phase color white white color translucent 7 orbital " + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3]);
   setZoom();
}


function jmolShowIsosurfaceShell(numPoints, radius) {
    if(isNaN(numPoints))
      numPoints = DEFAULT_NUM_POINTS
      
   if(isNaN(radius))
      radius = 0;
      
   jmolScript("isosurface curOrbitalShell color density orbital " + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3] + " points " + numPoints + " " + radius + " 7; echo " + curOrbital[4] + "; javascript " + postCalcFunction + ";");
   setZoom();
}


function jmolShowCrossSection(plane, phased, resolution) {
   if(!plane)
      plane = DEFAULT_PLANE;

   if(!resolution)
      resolution = toJmolResolution(DEFAULT_CROSSSECTION_CONTOUR_PLANE_RESOLUTION);
     
     jmolScript("isosurface curOrbital delete; isosurface contour* delete; isosurface crossSection" + plane + " plane " + plane 
       + " fullplane " + (phased ? "phase " : "") + "resolution " + resolution + " atomicOrbital " + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3] + "; javascript " + postCalcFunction + ";");
   setZoom();
}


function jmolHideCrossSection(plane) {
   if(!plane)
      plane = DEFAULT_PLANE;

   jmolScript("isosurface crossSection" + plane + " delete");
}


function jmolShowCrossSection3D(plane, phased, scale3d, resolution) {
   if(!plane)
      plane = DEFAULT_PLANE;

   if(isNaN(resolution))
      resolution = toJmolResolution(DEFAULT_CROSSSECTION_CONTOUR_PLANE_RESOLUTION);

   if(isNaN(scale3d))
      scale3d = getScale3d();

   jmolScript("isosurface curOrbital map plane " + plane + " scale3d " + scale3d + " fullplane "
    + (phased ? "phase " : "") + "color translucent 0.9 resolution " + resolution + " atomicOrbital " + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3] + "; javascript " + postCalcFunction + ";")
   setZoom();
}


function jmolShowContour(plane, numLines, resolution) {
   if(!plane)
      plane = DEFAULT_PLANE;

   if(isNaN(numLines))
      numLines = DEFAULT_CONTOUR_NUM_LINES;

   if(isNaN(resolution))
      resolution = toJmolResolution(DEFAULT_CROSSSECTION_CONTOUR_PLANE_RESOLUTION);

   jmolScript("isosurface curOrbital delete; isosurface crossSection* delete; isosurface contour" + plane + " plane " + plane + " contour " + numLines + " resolution " + resolution + " atomicOrbital " + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3] + "; javascript " + postCalcFunction + ";");
   setZoom();
}


function jmolHideContour(plane) {
   jmolScript("isosurface crossSection" + plane + " delete");
}


function jmolShowContour3D(plane, numLines, resolution, scale3d) {
   if(!plane)
      plane = DEFAULT_PLANE;

   if(isNaN(numLines))
      numLines = DEFAULT_CONTOUR_NUM_LINES;

   if(isNaN(resolution))
      resolution = toJmolResolution(DEFAULT_CROSSSECTION_CONTOUR_PLANE_RESOLUTION);

   if(isNaN(scale3d))
      scale3d = getScale3d();

   jmolScript("isosurface curOrbital plane " + plane + " contour " + numLines + " scale3d " + scale3d + " fullplane resolution " + resolution + " atomicOrbital " + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3] + "; javascript " + postCalcFunction + ";")
   setZoom();
}


function jmolShowPlane(plane, opacity, color) {
   // Default opacity and color if not given
   if(!opacity)
      opacity = DEFAULT_PLANE_OPACITY;

   if(!color)
      color = DEFAULT_PLANE_COLOR;

   jmolScript("isosurface plane" + plane + " plane " + plane + " resolution " + DEFAULT_CROSSSECTION_CONTOUR_PLANE_RESOLUTION + " color " + color + " translucent " + opacity + "; javascript " + postCalcFunction + ";");
}


function jmolHidePlane(plane) {
   jmolScript("isosurface plane" + plane + " delete;");
}


function jmolShowSphericalNode(radius, numNode, transparencyPercent, color) {
   if(!color)
      color = DEFAULT_NODE_COLOR;
   if(isNaN(transparencyPercent))
      transparencyPercent = DEFAULT_NODE_TRANSPARENCY;

   jmolScript("isosurface spherical" + numNode + " resolution " + toJmolResolution(5) + " color " + color + " color translucent " + toJmolTransparency(transparencyPercent) + " sphere " + radius + ";");
}


function jmolHideSphericalNode(numNode) {
   jmolScript("isosurface spherical" + numNode + " delete;");
}


function jmolShowConicalNode(conicalConstant, numNode, transparencyPercent, color) {
   if(!color)
      color = DEFAULT_NODE_COLOR;
   if(isNaN(transparencyPercent))
      transparencyPercent = DEFAULT_NODE_TRANSPARENCY;
   if(conicalConstant == "0.0")
      jmolScript("draw conical" + numNode + " width " + getNodeWidth() + " scale " + getNodeScale() + " color translucent " + toJmolTransparency(transparencyPercent) + " " + color + " cylinder {0 0 -0.03} {0 0 0.03};");
   else
      jmolScript("isosurface conical" + numNode + " scale " + getNodeScale() + " resolution " + (1/+getNodeScale()) + " color translucent " + toJmolTransparency(transparencyPercent) + " " + color + " functionxy = \"" + conicalConstant + "*(x**2+y**2)**0.5\";");
}


function jmolHideConicalNode(numNode) {
   jmolScript("isosurface conical" + numNode + " delete; draw conical" + numNode + " delete;");
}


function jmolShowPlanarNode(angle, numNode, transparencyPercent, color) {
   // Angle argument must be in the form of "(r, theta, phi)"
   if(!color)
      color = DEFAULT_NODE_COLOR;
   if(isNaN(transparencyPercent))
      transparencyPercent = DEFAULT_NODE_TRANSPARENCY;

   jmolScriptWait("isosurface planar" + numNode + " scale " + getNodeScale() + " resolution 0.5 color " + color + " color translucent " + toJmolTransparency(transparencyPercent) + " plane @{plane" + angle + "};");
}


function jmolHidePlanarNode(numNode) {
   jmolScript("isosurface planar" + numNode + " delete;");
}


function jmolLoadModel(filename) {
   jmolDeleteIsosurfaces(false);
   jmolScript("isosurface dorb " + filename + "; javascript " + postCalcFunction + ";");
   setZoom();
}


function jmolIsPointillistDisplayed() {
   // If the cross-section and contour are to be displayed over the pointillist display
   // the length of the isosurface array will need to be gotten and then looped through to check
   // all the isosurfaces for a pointillist one since the pointillist isosurface may not be the at the first
   // index in the array
   //jmolGetPropertyAsArray("shapeinfo.Isosurface").length;

   // Force title to be a string via the String() function
   var title = String(jmolGetPropertyAsArray("shapeInfo.Isosurface[1]").title);

   return (title != undefined && title.indexOf("POINTS") != -1);
}


function jmolIsWireframeDisplayed() {
   var stateInfo = jmolGetPropertyAsString("stateinfo");

   return (stateInfo != undefined && stateInfo.indexOf("noFill mesh") != -1);
}


function jmolSetStringVariable(varName, value) {
   jmolScriptWait(varName + " = \"" + value + "\";");
}

function jmolSetIntVariable(varName, value) {
   jmolScriptWait(varName + " = " + value + ";");
}

function jmolSetBoolVariable(varName, value) {
   jmolScriptWait(varName + " = " + ((value) ? "true" : "false") + ";");
}





// These functions should only be called by other functions in this file

function updateName() {
   curOrbital[4] = getOrbitalName(curOrbital[0], curOrbital[1], curOrbital[2]);
}


function setZoom() {
  if(!isToScale) {
    switch(parseInt(jmolGetN())) {
       case 1:
          jmolChangeZoom(getZoomLevel());
          jmolScaleAxes(getAxesScale());
          break;
       case 2:
          jmolChangeZoom(getZoomLevel());
          jmolScaleAxes(getAxesScale());
          break;
       case 3:
          jmolChangeZoom(getZoomLevel());
          jmolScaleAxes(getAxesScale());
          break;
       case 4:
          jmolChangeZoom(getZoomLevel());
          jmolScaleAxes(getAxesScale());
          break;
       case 5:
          jmolChangeZoom(getZoomLevel());
          jmolScaleAxes(getAxesScale());
       default:
    }

  // The axis ticks scale needs changed as well
  if(displayedAxesTicks[0])
     jmolShowAxesTicks("x");
  if(displayedAxesTicks[1])
     jmolShowAxesTicks("y");
  if(displayedAxesTicks[2])
     jmolShowAxesTicks("z");
  }
}


function getZoomLevel() {
  switch(parseInt(jmolGetN())) {
    case 1:
      return 70.333*curOrbital[0] + 157.417*curOrbital[3];
    case 2:
      return 9.6*curOrbital[0] + 82.7*curOrbital[3];
    case 3:
      return 4.422*curOrbital[0] + 46.685*curOrbital[3];
    case 4:
      return 2.577*curOrbital[0] + 28.673*curOrbital[3];
    case 5:
      return 1.333*curOrbital[0] + 23.333*curOrbital[3];
    default:
      return 100;
  }
}


function getNodeScale() {
  switch(parseInt(jmolGetN())) {
      case 2:
         return 0.65;
      case 3:
         return 1.2;
      case 4:
         return 2.0;
      case 5:
         return 2.78;
      default:
  }

  // N = 1 has no nodes so just return default scale if current N = 1
  return 1;
}

function getNodeWidth() {
  switch(parseInt(jmolGetN())) {
      case 2:
         return 15;
      case 3:
         return 25;
      case 4:
         return 40;
      case 5:
         return 55;
      default:
  }

  // N = 1 has no nodes so just return default scale if current N = 1
  return 1;
}


function getScale3d() {
   switch(parseInt(jmolGetN())) {
      case 1:
         return 5;
      case 2:
         return 40;
      case 3:
         return 200;
      case 4:
         return 400;
      case 5:
         return 1500;
      default:
         return -1;
  }
}

function getAxesScale() {
   return curOrbital[0]*1.1;
}


function getAxesTicksScale() {
   var ticksScale = {1: 1.5, 2: 2, 3: 2.5, 4: 3.75, 5: 5};

   return ticksScale[jmolGetN()];
}

function getDefaultIsosurfaceCommand(cutoff, resolution, isWireframe, colorPhase) {
   // Default cutoff, colorPhase, and resolution if not given
   if(isNaN(cutoff))
      cutoff = DEFAULT_CUTOFF;

   if(isNaN(resolution))
      resolution = DEFAULT_RESOLUTION;

   if(!colorPhase)
      colorPhase = DEFAULT_ISOSURFACE_COLORPHASE;

   // If this is being used for wireframe, do not change the resolution based on N
   if(!isWireframe)
      resolution -= curOrbital[0];

   // Note that the resolution is set at DEFAULT_RESOLUTION - n
   // This way the quicker to calculate orbitals (n = 1,2,3) have a higher resolution and the slower to calculate orbitals (4,5)
   // have a slightly lower resolution but are also quick to calculate
   return "isosurface curOrbital resolution " + resolution + " cutoff " + cutoff + " phase color " + colorPhase + " atomicOrbital " + curOrbital[0] + " " + curOrbital[1] + " " + curOrbital[2] + " " + curOrbital[3];
}


function getOrbitalName(n, l, m) {
   var names = {"100":  "1s",

                "200":  "2s",

                "21-1": "2p<sub>y</sub>",
                "210":  "2p<sub>z</sub>",
                "211":  "2p<sub>x</sub>",

                "300":  "3s",

                "31-1": "3p<sub>y</sub>",
                "310":  "3p<sub>z</sub>",
                "311":  "3p<sub>x</sub>",

                "32-2": "3d<sub>xy</sub>",
                "32-1": "3d<sub>yz</sub>",
                "320":  "3d<sub>z<sup>2</sup></sub>",
                "321":  "3d<sub>xz</sub>",
                "322":  "3d<sub>x<sup>2</sup>-y<sup>2</sup></sub>",

                "400":  "4s",

                "41-1": "4p<sub>y</sub>",
                "410":  "4p<sub>z</sub>",
                "411":  "4p<sub>x</sub>",

                "42-2": "4d<sub>xy</sub>",
                "42-1": "4d<sub>yz</sub>",
                "420":  "4d<sub>z</sub>2",
                "421":  "4d<sub>xz</sub>",
                "422":  "4d<sub>x<sup>2</sup>-y<sup>2</sup></sub>",

                "43-3": "4f<sub>y(3x<sup>2</sup>-y<sup>2</sup>)</sub>",
                "43-2": "4f<sub>xyz</sub>",
                "43-1": "4f<sub>yz<sup>2</sup></sub>",
                "430":  "4f<sub>z<sup>3</sup></sub>",
                "431":  "4f<sub>xz<sup>2</sup></sub>",
                "432":  "4f<sub>z(x<sup>2</sup>-y<sup>2</sup>)</sub>",
                "433":  "4f<sub>x(x<sup>2</sup>-3y<sup>2</sup>)</sub>",

                "500":  "5s",

                "51-1": "5p<sub>y</sub>",
                "510":  "5p<sub>z</sub>",
                "511":  "5p<sub>x</sub>",

                "52-2": "5d<sub>xy</sub>",
                "52-1": "5d<sub>yz</sub>",
                "520":  "5d<sub>z</sub>2",
                "521":  "5d<sub>xz</sub>",
                "522":  "5d<sub>x<sup>2</sup>-y<sup>2</sup></sub>",

                "53-3": "5f<sub>y(3x<sup>2</sup>-y<sup>2</sup>)</sub>",
                "53-2": "5f<sub>xyz</sub>",
                "53-1": "5f<sub>yz<sup>2</sup></sub>",
                "530":  "5f<sub>z<sup>3</sup></sub>",
                "531":  "5f<sub>xz<sup>2</sup></sub>",
                "532":  "5f<sub>z(x<sup>2</sup>-y<sup>2</sup>)</sub>",
                "533":  "5f<sub>x(x<sup>2</sup>-3y<sup>2</sup>)</sub>",

                "54-4": "5g<sub>xy(x<sup>2</sup>-y<sup>2</sup>)</sub>",
                "54-3": "5g<sub>zy<sup>3</sup>",
                "54-2": "5g<sub>z<sup>2</sup>xy</sub>",
                "54-1": "5g<sub>z<sup>3</sup>y</sub>",
                "540":  "5g<sub>z<sup>4</sup></sub>",
                "541":  "5g<sub>z<sup>3</sup>x</sub>",
                "542":  "5g<sub>z<sup>2</sup>(x<sup>2</sup>-y<sup>2</sup>)</sub>",
                "543":  "5g<sub>zx<sup>3</sup></sub>",
                "544":  "5g<sub>x<sup>4</sup>+y<sup>4</sup></sub>"
	};

   return names[n + "" + l + "" + m];
}